/*
 * ExactSubtreeMatchingAutomaton.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_SUBTREE_MATCHING_AUTOMATON_H__
#define _EXACT_SUBTREE_MATCHING_AUTOMATON_H__

#include <automaton/Automaton.h>
#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/TA/NFTA.h>
#include <tree/Tree.h>

namespace arbology {

namespace exact {

class ExactSubtreeMatchingAutomaton : public tree::VisitableTreeBase::const_visitor_type {
public:
	ExactSubtreeMatchingAutomaton() {}

	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static automaton::Automaton construct(const tree::Tree& pattern);

	static automaton::InputDrivenNPDA construct(const tree::PrefixRankedTree& pattern);
	static automaton::NFTA construct(const tree::RankedTree& pattern);
private:
	void Visit(void*, const tree::RankedTree& pattern) const;
	void Visit(void*, const tree::RankedPattern& subject) const;
	void Visit(void*, const tree::PrefixRankedTree& pattern) const;
	void Visit(void*, const tree::PrefixRankedBarTree& pattern) const;
	void Visit(void*, const tree::PrefixRankedPattern& pattern) const;
	void Visit(void*, const tree::PrefixRankedBarPattern& pattern) const;
	void Visit(void*, const tree::UnrankedTree& pattern) const;
	void Visit(void*, const tree::UnrankedPattern& subject) const;

	static const ExactSubtreeMatchingAutomaton EXACT_SUBTREE_MATCHING_AUTOMATON;
};

} /* namespace exact */

} /* namespace arbology */

#endif /* _EXACT_SUBTREE_MATCHING_AUTOMATON_H__ */
