/*
 * ToRegExpAlgebraic.h
 *
 *  Created on: 11. 2. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATON_TO_REG_EXP_ALGEBRAIC_H_
#define AUTOMATON_TO_REG_EXP_ALGEBRAIC_H_

#include <deque>
#include <map>
#include <queue>

#include <regexp/RegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>
#include <automaton/Automaton.h>

namespace automaton {

namespace convert {

// http://cs.stackexchange.com/questions/2016/how-to-convert-finite-automata-to-regular-expressions
// https://github.com/ferno/greenery/blob/bcc0a136335edbe94cd7725fc6e8cce0268d850c/fsm.py

/**
 * Converts FA to RE using Brzozowski's algebraic method using right regular equations.
 * Source : Melichar 2.122
 */
class ToRegExpAlgebraic : public automaton::VisitableConstFSMBase {
public:
	ToRegExpAlgebraic() {}

	/**
	 * Performs conversion.
	 * @return regular expression equivalent to input automaton.
	 */
	static regexp::RegExp convert(const automaton::Automaton& automaton);

	static regexp::UnboundedRegExp convert(const automaton::EpsilonNFA& automaton);
	static regexp::UnboundedRegExp convert(const automaton::MultiInitialStateNFA& automaton);
	static regexp::UnboundedRegExp convert(const automaton::NFA& automaton);
	static regexp::UnboundedRegExp convert(const automaton::DFA& automaton);

private:
	using automaton::VisitableConstFSMBase::Visit;

	void Visit(void*, const automaton::EpsilonNFA&) const;
	void Visit(void*, const automaton::MultiInitialStateNFA&) const;
	void Visit(void*, const automaton::NFA&) const;
	void Visit(void*, const automaton::DFA&) const;
	void Visit(void*, const automaton::ExtendedNFA&) const;
	void Visit(void*, const automaton::CompactNFA&) const;

	static const ToRegExpAlgebraic TO_REG_EXP_ALGEBRAIC;
};

} /* namespace convert */

} /* namespace automaton */

#endif /* AUTOMATON_TO_REG_EXP_ALGEBRAIC_H_ */
