/*
 * StringContainerConverter.h
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#ifndef _STRING_CONTAINER_CONVERTER__H_
#define _STRING_CONTAINER_CONVERTER__H_

#include <string/LinearString.h>
#include <string/CyclicString.h>
#include <string/Epsilon.h>
#include <string/String.h>
#include <exception/AlibException.h>

namespace common {

/* R is container<T>
 * S is container<String>
 * T is Epsilon, LinearString, or CyclicString
 */
template<class R, class S, class T>
class StringContainerConverter : public string::VisitableStringBase::const_visitor_type {
public:
	StringContainerConverter() {}

	/**
	 * Performs conversion from contaier<String> to container<T>.
	 * @return container<T>.
	 */
	static R convert(const S& containerOfStrings);

private:
	void Visit(void*, const string::Epsilon& text) const;
	void Visit(void*, const string::LinearString& text) const;
	void Visit(void*, const string::CyclicString& text) const;

	static const StringContainerConverter<R, S, T> STRING_CONTAINER_CONVERTER;
};

template<class R, class S, class T>
R StringContainerConverter<R, S, T>::convert(const S& containerOfStrings) {
	R result;
	auto inserter = std::inserter(result, result.end());
	for(const string::String& elem : containerOfStrings) {
		T* out;
		elem.getData().Accept((void*) &out, StringContainerConverter<R, S, T>::STRING_CONTAINER_CONVERTER);
		inserter = std::move(*out);
		delete out;
	}
	return result;
}



template<class R, class S, class T,
	typename std::enable_if< std::is_constructible<T, string::Epsilon>::value >::type* = nullptr >
inline void VisitHelper(void* data, const string::Epsilon& text) {
	T* & out = *((T**) data);
	out = new T(text);
}

template<class R, class S, class T,
	typename std::enable_if< ! std::is_constructible<T, string::Epsilon>::value >::type* = nullptr >
inline void VisitHelper(void*, const string::Epsilon&) {
	throw exception::AlibException("Unsupported string type Epsilon");
}

template<class R, class S, class T>
void StringContainerConverter<R, S, T>::Visit(void* data, const string::Epsilon& text) const {
	VisitHelper<R, S, T>(data, text);
}



template<class R, class S, class T,
	typename std::enable_if< std::is_constructible<T, string::LinearString>::value >::type* = nullptr >
inline void VisitHelper(void* data, const string::LinearString& text) {
	T* & out = *((T**) data);
	out = new T(text);
}

template<class R, class S, class T,
	typename std::enable_if< ! std::is_constructible<T, string::LinearString>::value >::type* = nullptr >
inline void VisitHelper(void*, const string::LinearString&) {
	throw exception::AlibException("Unsupported string type LinearString");
}

template<class R, class S, class T>
void StringContainerConverter<R, S, T>::Visit(void* data, const string::LinearString& text) const {
	VisitHelper<R, S, T>(data, text);
}



template<class R, class S, class T,
	typename std::enable_if< std::is_constructible<T, string::CyclicString>::value >::type* = nullptr >
inline void VisitHelper(void* data, const string::CyclicString& text) {
	T* & out = *((T**) data);
	out = new T(text);
}

template<class R, class S, class T,
	typename std::enable_if< ! std::is_constructible<T, string::CyclicString>::value >::type* = nullptr >
inline void VisitHelper(void*, const string::CyclicString&) {
	throw exception::AlibException("Unsupported string type CyclicString");
}

template<class R, class S, class T>
void StringContainerConverter<R, S, T>::Visit(void* data, const string::CyclicString& text) const {
	VisitHelper<R, S, T>(data, text);
}



template<class R, class S, class T>
const StringContainerConverter<R, S, T> StringContainerConverter<R, S, T>::STRING_CONTAINER_CONVERTER;

} /* namespace common */

#endif /* _STRING_CONTAINER_CONVERTER__H_ */
