/*
 * BadCharacterShiftTable.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka, Jan Travnicek
 */

#include "BadCharacterShiftTable.h"
#include <string/StringAlphabetGetter.h>

#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <alphabet/Symbol.h>

namespace stringology {

namespace exact {

std::map<alphabet::Symbol, size_t> BadCharacterShiftTable::bcs(const string::String& pattern, const std::set<alphabet::Symbol>& alphabet) {
	std::pair<std::map<alphabet::Symbol, size_t>, const std::set<alphabet::Symbol>*> data;
	data.second = &alphabet;
	pattern.getData().Accept((void*) &data, BadCharacterShiftTable::BAD_CHARACTER_SHIFT);
	return data.first;
}

std::map<alphabet::Symbol, size_t> BadCharacterShiftTable::bcs(const string::String& pattern) {
	return bcs(pattern, string::StringAlphabetGetter::getAlphabet(pattern));
}

std::map<alphabet::Symbol, size_t> BadCharacterShiftTable::bcs(const string::LinearString& pattern, const std::set<alphabet::Symbol>& alphabet) {
	std::map<alphabet::Symbol, size_t> bcs;

	/* Initialization of BCS to the length of the needle. */
	for(const auto& symbol : alphabet)
		bcs.insert(std::make_pair(symbol, pattern.getContent().size()));

	/* Filling out BCS, ignoring last character. */
	for(size_t i = 0; i < pattern.getContent().size() - 1; i++)
		bcs[pattern.getContent().at(i)] = pattern.getContent().size() - i - 1;

	/*
	for(const auto& kv: bcs)
		std::cout << std::string(kv.first) << " " << kv.second << std::endl;
	for(const auto& s: string.getContent())
		std::cout << std::string(s);std::cout << std::endl;
	*/

	return bcs;
}

std::map<alphabet::Symbol, size_t> BadCharacterShiftTable::bcs(const string::LinearString& pattern) {
	return bcs(pattern, pattern.getAlphabet());
}

void BadCharacterShiftTable::Visit(void*, const string::Epsilon&) const {
	throw exception::AlibException("Unsupported string type Epsilon");
}

void BadCharacterShiftTable::Visit(void* data, const string::LinearString& pattern) const {
	std::pair<std::map<alphabet::Symbol, size_t>, const std::set<alphabet::Symbol>*> & res = *((std::pair<std::map<alphabet::Symbol, size_t>, const std::set<alphabet::Symbol>*>*) data);
	res.first = this->bcs(pattern, *res.second);
}

void BadCharacterShiftTable::Visit(void*, const string::CyclicString&) const {
	throw exception::AlibException("Unsupported string type CyclicString");
}

const BadCharacterShiftTable BadCharacterShiftTable::BAD_CHARACTER_SHIFT;

} /* namespace exact */

} /* namespace stringology */
