/*
 * BoyerMooreHorspool.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka
 */

#ifndef _BOYER_MOORE_HORSPOOL_H
#define _BOYER_MOORE_HORSPOOL_H

#include <string/String.h>

namespace stringology {

namespace exact {

/**
 * Implementation of BMH for MI(E+\eps)-EVY course 2014
 * To get rid of zeros in BCS table we ignore last haystack character
 */
class BoyerMooreHorspool : public string::VisitableStringBase::const_same_visitor_type {
public:
	BoyerMooreHorspool() {}

	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::set<unsigned> match(const string::String& subject, const string::String& pattern);

	static std::set<unsigned> match(const string::LinearString& subject, const string::LinearString& pattern);
private:
	void Visit(void*, const string::Epsilon& subject, const string::Epsilon& pattern) const;
	void Visit(void*, const string::LinearString& subject, const string::LinearString& pattern) const;
	void Visit(void*, const string::CyclicString& subject, const string::CyclicString& pattern) const;

	static const BoyerMooreHorspool BOYER_MOORE_HORSPOOL;
};

} /* namespace exact */

} /* namespace stringology */

#endif /* _BOYER_MOORE_HORSPOOL_H */
