/*
 * ExactSubsequenceAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactSubsequenceAutomaton.h"
#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactSubsequenceAutomaton::construct(const string::String& text) {
	automaton::Automaton* out = NULL;
	text.getData().Accept((void*) &out, ExactSubsequenceAutomaton::EXACT_SUBSEQUENCE_AUTOMATON);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

automaton::DFA ExactSubsequenceAutomaton::construct(const string::LinearString& text) {
	std::map<alphabet::Symbol, int> f;
	for(const alphabet::Symbol& symbol : text.getAlphabet()) {
		f[symbol] = 0;
	}

	automaton::DFA res(automaton::State(0));
	res.addFinalState(automaton::State(0));
	res.setInputAlphabet(text.getAlphabet());

	int i = 1;
	for(const alphabet::Symbol& symbol : text.getContent()) {
		res.addState(automaton::State(i));
		res.addFinalState(automaton::State(i));

		for(int j = f[symbol]; j < i; j++) {
			res.addTransition(automaton::State(j), symbol, automaton::State(i));
		}
		f[symbol] = i;
		i++;
	}
	return res;
}

void ExactSubsequenceAutomaton::Visit(void* data, const string::Epsilon& text) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->construct(string::LinearString(text)));
}

void ExactSubsequenceAutomaton::Visit(void* data, const string::LinearString& text) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->construct(text));
}

void ExactSubsequenceAutomaton::Visit(void*, const string::CyclicString&) const {
	throw exception::AlibException("Unsupported string type CyclicString");
}

const ExactSubsequenceAutomaton ExactSubsequenceAutomaton::EXACT_SUBSEQUENCE_AUTOMATON;

} /* namespace exact */

} /* namespace stringology */
